#include "CallBack.h"
#include "ErrorCollector.h"
#include "InternalException.h"
#include <vector>
#include <sstream>
#include <iostream>

namespace aimms{

    CallBack::CallBack(class ErrorCollector* ec, const char* projectName)
        : m_Collector(ec)
        , m_ProjectName(projectName)
    {
    }
    CallBack::~CallBack()
    {
    }
    void CallBack::onMessages(std::vector<AimmsMessage>& messages )
    {
        // store all messages in the collector, 
        // and throw when there is actually an error.

        std::cerr << "OnMessages :";
        if (messages.size()> 0){
            std::vector<AimmsMessage>::iterator it = messages.begin();
            std::vector<AimmsMessage>::iterator end = messages.end();

            std::ostringstream oneLongMessage;
            oneLongMessage<< m_ProjectName << ": " ;
            AimmsMessageCode code = AM_OK;
            while (it != end){
                oneLongMessage << it->message << '\t';        
                if (code <= AM_Warning){
                  code = it->code;  
                }
                ++it;
                
            }
            std::cerr << oneLongMessage.str() << std::endl;
            m_Collector->addError(oneLongMessage.str().c_str());

            if (code >AM_Warning){
                throw InternalException(messages[0].code);
            }

        }
    }

    void CallBack::addError(const char* error)
    {
        std::ostringstream projectError;
        projectError<< m_ProjectName << ": " << error ;
        m_Collector->addError(projectError.str().c_str());
    }



}