#include "Iterator.h"

#include "OrdinalDenseLevel.h"
#include "OrdinalSparseLevel.h"
#include "IntegerDenseLevel.h"
#include "IntegerSparseLevel.h"

#include "aimms/Include.h"

namespace aimms{

    Iterator::Iterator()
        : m_Tuple(0) 
        , m_Outerlevel(0)
    {
    }

    Iterator::Iterator(IMultiDimData* par)
        : m_Tuple(par->createTuple()) 
        , m_Outerlevel(0)
    {    
    }

    Iterator::~Iterator()
    {
        m_Tuple->close();
        delete m_Outerlevel;
    }

    void Iterator::SetOrdinalLevelsCMO(int** slices, int* slicecards, int* cards)
    {
         for (int lvl=0; lvl<m_Tuple->getDimension(); ++lvl) {
            if ( slices && slices[lvl] ) {
                m_Outerlevel = new OrdinalSparseLevel(m_Outerlevel,cards[lvl],m_Tuple->getElement(lvl),slicecards[lvl],slices[lvl]);
            } else {
                m_Outerlevel = new OrdinalDenseLevel(m_Outerlevel,cards[lvl],m_Tuple->getElement(lvl));
            }
        }
    }
    void Iterator::SetIntegerLevelsCMO(int** slices, int* slicecards, int* firsts, int* lasts)
    {
         for (int lvl=0; lvl<m_Tuple->getDimension(); ++lvl) {
            if ( slices && slices[lvl] ) {
                m_Outerlevel = new IntegerSparseLevel(m_Outerlevel,firsts[lvl],lasts[lvl],m_Tuple->getElement(lvl),slicecards[lvl],slices[lvl]);
            } else {
                m_Outerlevel = new IntegerDenseLevel(m_Outerlevel,firsts[lvl],lasts[lvl],m_Tuple->getElement(lvl));
            }
        }
    } 

    void Iterator::SetOrdinalLevelsRMO(int** slices, int* slicecards, int* cards)
    {
         for (int lvl=m_Tuple->getDimension()-1; lvl>=0; --lvl) {
            if ( slices && slices[lvl] ) {
                m_Outerlevel = new OrdinalSparseLevel(m_Outerlevel,cards[lvl],m_Tuple->getElement(lvl),slicecards[lvl],slices[lvl]);
            } else {
                m_Outerlevel = new OrdinalDenseLevel(m_Outerlevel,cards[lvl],m_Tuple->getElement(lvl));
            }
        }
    }

    void Iterator::SetIntegerLevelsRMO(int** slices, int* slicecards, int* firsts, int* lasts)
    {
         for (int lvl=m_Tuple->getDimension()-1; lvl>=0; --lvl) {
            if ( slices && slices[lvl] ) {
                m_Outerlevel = new IntegerSparseLevel(m_Outerlevel,firsts[lvl],lasts[lvl],m_Tuple->getElement(lvl),slicecards[lvl],slices[lvl]);
            } else {
                m_Outerlevel = new IntegerDenseLevel(m_Outerlevel,firsts[lvl],lasts[lvl],m_Tuple->getElement(lvl));
            }
        }
    } 
}