#ifndef _INCLUDE_aimms_SliceRetriever
#define _INCLUDE_aimms_SliceRetriever

#include "Iterator.h"
#include "RetrieveDataLevel.h"

namespace aimms
{
   // This class assigns data from an Aimms Identifier to a slice. 
    // All data in the slice is set to 0.0, and then the Aimms data is iterated (currently with next) to fill the slice.
    // The class must be initialized (once!) with one of the 'level setters'
    // and then can be used (once!) by calling iterate
    // e.g. SetOrdinalLevelsCMO().
    template<typename T>
    class SliceRetriever : public Iterator
    {
    public:
        SliceRetriever(IMultiDimData* par,T* values, int fronttuples)
            : Iterator(par)
            , m_Iterator(par->createIterator(fronttuples))
        {
            m_Outerlevel = new RetrieveDataLevel<T>(values);
        }
        virtual ~SliceRetriever() 
        {
            m_Iterator->close();
        }
        void clearSlice()
        {     
            // Set the entire slice to 0.0
            m_Outerlevel->HandleSliceNoTuple(0);
        }
        void retrieveSlice()
        {
            // Use the aimms iterator and skip anything which is not in the slices
            while (m_Iterator->next()) {
                m_Iterator->getTuple(m_Tuple);
                m_Outerlevel->Apply(m_Iterator->asDouble(),0);
            }
        }
    private:
        class IIterator* m_Iterator;
    };
}
#endif